@@ -1,10 +1,10 @@
 /*
     fileio
-    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "fileio.h"
 #include <time.h>
@@ -27,19 +26,21 @@
 #if GLIB_CHECK_VERSION(2,6,0)
 # include <glib/gstdio.h>
 #endif /* GLIB_CHECK_VERSION(2,6,0) */
-#ifdef G_OS_WIN32
+#if defined (G_OS_WIN32)
 # include <shlwapi.h>
-#endif /* G_OS_WIN32 */
+#elif GLIB_CHECK_VERSION(2,16,0)
+# include <gio/gio.h>
+#endif /* GLIB_CHECK_VERSION(2,16,0) */
 
 
 struct _FileIO
 {
   gint counter;
+  gchar *file;
 #ifdef G_OS_WIN32
   HANDLE hFile;
 #else /* not G_OS_WIN32 */
   int fd;
-  gchar *file;
   guint share;
 #endif /* not G_OS_WIN32 */
 };
@@ -533,7 +534,6 @@ fileio_open (const gchar *file,
       g_free (dir);
     }
 #ifdef G_OS_WIN32
-  g_free (path);
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -544,7 +544,10 @@ fileio_open (const gchar *file,
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       if (!utf8str)
-        return NULL;
+        {
+          g_free (path);
+          return NULL;
+        }
       lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       hFile = CreateFileW (lpszFile, access, share, NULL, mode,
                                                 FILE_ATTRIBUTE_NORMAL, NULL);
@@ -573,9 +576,13 @@ fileio_open (const gchar *file,
     }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (hFile == INVALID_HANDLE_VALUE)
-    return NULL;
+    {
+      g_free (path);
+      return NULL;
+    }
   fio = g_malloc (sizeof (FileIO));
   fio->counter = 1;
+  fio->file = path;
   fio->hFile = hFile;
 #else /* not G_OS_WIN32 */
   /* ja:フラグ設定 */
@@ -650,8 +657,8 @@ fileio_open (const gchar *file,
     }
   fio = g_malloc (sizeof (FileIO));
   fio->counter = 1;
-  fio->fd = fd;
   fio->file = path;
+  fio->fd = fd;
   fio->share = share;
 # ifdef USE_THREAD
   G_LOCK (critical);
@@ -711,6 +718,7 @@ fileio_unref (FileIO *fio)
       if (fio->counter <= 0)
         {
           result = CloseHandle (fio->hFile);
+          g_free (fio->file);
           g_free (fio);
         }
       else
@@ -747,6 +755,16 @@ fileio_unref (FileIO *fio)
 }
 
 
+/*  ja:ファイルの名前を取得する
+    fio,ファイルポインタ
+    RET,名前(フルパス)                                                      */
+const gchar *
+fileio_getname (FileIO *fio)
+{
+  return (const gchar *)(fio ? fio->file : NULL);
+}
+
+
 /*  ja:ファイルから読み込む
        fio,ファイルポインタ
       data,バッファ
@@ -824,18 +842,14 @@ fileio_seek (FileIO        *fio,
              const guint    whence)
 {
 #ifdef G_OS_WIN32
-# ifdef G_HAVE_GINT64
   LONG lHigh, lLow;
 
   if (!fio)
     return -1;
   lHigh = offset >> 32;
   lLow = SetFilePointer (fio->hFile, (LONG)offset, &lHigh, whence);
-  return lLow != 0xFFFFFFFF || GetLastError () == NO_ERROR
+  return lLow != 0xffffffff || GetLastError () == NO_ERROR
                                 ? ((gint64)lHigh << 32) | (guint32)lLow : -1;
-# else /* not G_HAVE_GINT64 */
-  return SetFilePointer (fio->hFile, offset, NULL, whence);
-# endif /* not G_HAVE_GINT64 */
 #else /* not G_OS_WIN32 */
   gssize result;
 
@@ -861,17 +875,13 @@ goffset
 fileio_size (FileIO *fio)
 {
 #ifdef G_OS_WIN32
-# ifdef G_HAVE_GINT64
   DWORD dwHigh, dwLow;
 
   if (!fio)
     return -1;
   dwLow = GetFileSize (fio->hFile, &dwHigh);
-  return dwLow != 0xFFFFFFFF || GetLastError () == NO_ERROR
+  return dwLow != 0xffffffff || GetLastError () == NO_ERROR
                                 ? ((gint64)dwHigh << 32) | (guint32)dwLow : -1;
-# else /* not G_HAVE_GINT64 */
-  return GetFileSize (fio->hFile, NULL);
-# endif /* not G_HAVE_GINT64 */
 #else /* not G_OS_WIN32 */
   goffset offset, leng;
 
@@ -1482,7 +1492,7 @@ fileio_settime (const gchar *file,
 }
 
 
-/*  ja:ファイルの時刻を変換する
+/*  ja:システム時刻を取得する
     ftime,ファイルの時刻
     stime,システム時間                                                      */
 void
@@ -1515,6 +1525,42 @@ fileio_getsystime (FileIOTime    *ftime,
 }
 
 
+/*  ja:システム時刻を設定する
+    ftime,ファイルの時刻
+    stime,システム時間                                                      */
+void
+fileio_setsystime (FileIOTime    *ftime,
+                   FileIOSysTime *stime)
+{
+#ifdef G_OS_WIN32
+  SYSTEMTIME st;
+
+  st.wYear         = stime->year;
+  st.wMonth        = stime->mon;
+  st.wDayOfWeek    = stime->wday;
+  st.wDay          = stime->mday;
+  st.wHour         = stime->hour;
+  st.wMinute       = stime->min;
+  st.wSecond       = stime->sec;
+  st.wMilliseconds = 0;
+  SystemTimeToFileTime (&st, ftime);
+#else /* not G_OS_WIN32 */
+  struct tm gmt;
+
+  gmt.tm_sec   = stime->sec;
+  gmt.tm_min   = stime->min;
+  gmt.tm_hour  = stime->hour;
+  gmt.tm_mday  = stime->mday;
+  gmt.tm_mon   = stime->mon - 1;
+  gmt.tm_year  = stime->year - 1900;
+  gmt.tm_wday  = stime->wday;
+  gmt.tm_yday  = 0;
+  gmt.tm_isdst = -1;
+  *ftime = mktime (&gmt);
+#endif /* not G_OS_WIN32 */
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:応用ファイル関数群                                                       *
@@ -1528,53 +1574,42 @@ GdkPixbuf *
 fileio_extract_icon (const gchar *file,
                      GtkIconSize  icon_size)
 {
-#if defined (G_OS_WIN32)
+#if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   gint icon_x, icon_y;
   GdkPixbuf *pixbuf = NULL;
+# ifdef G_OS_WIN32
   HICON hIcon = NULL;
   ICONINFO ii;
   UINT uFlags;
-# if GLIB_CHECK_VERSION(2,2,0)
-  GtkSettings *settings;
-# endif /* GLIB_CHECK_VERSION(2,2,0) */
-# if ! GLIB_CHECK_VERSION(2,6,0)
+#  if ! GLIB_CHECK_VERSION(2,6,0)
   LPTSTR lpszFile;
   SHFILEINFO shfi;
-#  ifdef UNICODE
+#   ifdef UNICODE
   gchar *utf8str;
-#  endif /* UNICODE */
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#   endif /* UNICODE */
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# endif /* G_OS_WIN32 */
 
-  if (!file)
+  if (!gtk_icon_size_lookup (icon_size, &icon_x, &icon_y))
+    switch (icon_size)
+      {
+        case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
+        case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
+        case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
+        case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
+        case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
+        case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
+        default:                          icon_x =  0; icon_y =  0;
+      }
+  if (!file || icon_x <= 0 || icon_y <= 0)
+#endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
     return NULL;
-  switch (icon_size)
-    {
-      case GTK_ICON_SIZE_MENU:          icon_x = 16; icon_y = 16; break;
-      case GTK_ICON_SIZE_SMALL_TOOLBAR: icon_x = 18; icon_y = 18; break;
-      case GTK_ICON_SIZE_LARGE_TOOLBAR: icon_x = 24; icon_y = 24; break;
-      case GTK_ICON_SIZE_BUTTON:        icon_x = 20; icon_y = 20; break;
-      case GTK_ICON_SIZE_DND:           icon_x = 32; icon_y = 32; break;
-      case GTK_ICON_SIZE_DIALOG:        icon_x = 48; icon_y = 48; break;
-      default:                          icon_x =  0; icon_y =  0;
-    }
-# if GLIB_CHECK_VERSION(2,2,0)
-  settings = gtk_settings_get_default ();
-  if (settings)
-    {
-      gint w, h;
-
-      if (gtk_icon_size_lookup_for_settings (settings, icon_size, &w, &h))
-        {
-          icon_x = w;
-          icon_y = h;
-        }
-    }
+#if defined (G_OS_WIN32)
   uFlags = ABS (GetSystemMetrics (SM_CXSMICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                 < ABS (GetSystemMetrics (SM_CXICON) - icon_x)
                                 + ABS (GetSystemMetrics (SM_CYSMICON) - icon_y)
                                         ? SHGFI_SMALLICON : SHGFI_LARGEICON;
-# endif /* GLIB_CHECK_VERSION(2,2,0) */
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -1683,12 +1718,7 @@ fileio_extract_icon (const gchar *file,
     }
   if (hIcon)
     DestroyIcon (hIcon);
-  return pixbuf;
-#elif GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
-  GdkPixbuf *pixbuf = NULL;
-
-  if (!file)
-    return NULL;
+#elif GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   if (g_file_test (file, G_FILE_TEST_IS_DIR))
     {
       const gchar *home;
@@ -1707,139 +1737,119 @@ fileio_extract_icon (const gchar *file,
     }
   else
     {
-      gchar *text;
-      gssize length;
+      gchar *mime_type = NULL;
+# if GLIB_CHECK_VERSION(2,16,0)
+      gboolean uncertain;
+      gchar *content_type;
 
-      text = fileio_load (MIMEFILE, &length);
-      if (text)
+      content_type = g_content_type_guess (file, NULL, 0, &uncertain);
+      if (content_type)
         {
-          gchar **mimes, *utf8str, *mime_type = NULL;
-          gint i;
-
-          text = g_realloc (text, length + 1);
-          text[length] = '\0';
-          utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
-          mimes = g_strsplit (text, "\n", 0);
-          for (i = 0; mimes[i]; i++)
+          mime_type = g_content_type_get_mime_type (content_type);
+          g_free (content_type);
+        }
+# endif /* GLIB_CHECK_VERSION(2,16,0) */
+# ifdef MIMEFILE
+#  if GLIB_CHECK_VERSION(2,16,0)
+      if (!mime_type)
+        {
+#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+          gchar *text;
+          gssize length;
+          text = fileio_load (MIMEFILE, &length);
+          if (text)
             {
-              gchar **globs;
+              gchar **mimes, *utf8str;
+              gint i;
 
-              globs = g_strsplit (mimes[i], G_SEARCHPATH_SEPARATOR_S, 0);
-              if (globs[0])
+              text = g_realloc (text, length + 1);
+              text[length] = '\0';
+              utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+              mimes = g_strsplit (text, "\n", 0);
+              for (i = 0; mimes[i]; i++)
                 {
-                  gint j;
+                  gchar **globs;
 
-                  for (j = 1; globs[j]; j++)
-                    if (g_pattern_match_simple (globs[j], utf8str))
-                      {
-                        mime_type = g_strdup (globs[0]);
-                        break;
-                      }
-                }
-              g_strfreev (globs);
-              if (mime_type)
-                break;
-            }
-          g_strfreev (mimes);
-          g_free (utf8str);
-          g_free (text);
-          if (mime_type)
-            {
-              gchar *icon_name, *separator;
-
-              icon_name = g_strconcat ("gnome-mime-", mime_type, NULL);
-              g_free (mime_type);
-              separator = g_strchr (icon_name, '/');
-              if (separator)
-                {
-                  GtkIconTheme *icon_theme;
-
-                  *separator = '-';
-                  icon_theme = gtk_icon_theme_get_default ();
-                  if (icon_theme)
+                  globs = g_strsplit (mimes[i], G_SEARCHPATH_SEPARATOR_S, 0);
+                  if (globs[0])
                     {
-                      gboolean result;
+                      gint j;
 
+                      for (j = 1; globs[j]; j++)
+                        if (g_pattern_match_simple (globs[j], utf8str))
+                          {
+                            mime_type = g_strdup (globs[0]);
+                            break;
+                          }
+                    }
+                  g_strfreev (globs);
+                  if (mime_type)
+                    break;
+                }
+              g_strfreev (mimes);
+              g_free (utf8str);
+              g_free (text);
+            }
+#  if GLIB_CHECK_VERSION(2,16,0)
+        }
+#  endif /* GLIB_CHECK_VERSION(2,16,0) */
+# endif /* MIMEFILE */
+      if (mime_type)
+        {
+          gchar *icon_name, *separator;
+
+          icon_name = g_strconcat ("gnome-mime-", mime_type, NULL);
+          g_free (mime_type);
+          separator = g_strchr (icon_name, '/');
+          if (separator)
+            {
+              GtkIconTheme *icon_theme;
+
+              *separator = '-';
+              icon_theme = gtk_icon_theme_get_default ();
+              if (icon_theme)
+                {
+                  gboolean result;
+
+                  result = gtk_icon_theme_has_icon (icon_theme, icon_name);
+                  if (!result)
+                    {
+                      *separator = '\0';
                       result = gtk_icon_theme_has_icon (icon_theme, icon_name);
-                      if (!result)
-                        {
-                          *separator = '\0';
-                          result = gtk_icon_theme_has_icon (icon_theme,
-                                                                    icon_name);
-                        }
-                      if (result)
-                        {
-                          gint icon_x, icon_y;
+                    }
+                  if (result)
+                    {
 # if GLIB_CHECK_VERSION(2,6,0)
-                          gint *sizes;
-# endif /* not GLIB_CHECK_VERSION(2,6,0) */
-                          GtkSettings *settings;
+                      gint *sizes;
 
-                          switch (icon_size)
-                            {
-                              case GTK_ICON_SIZE_MENU:
-                                icon_x = icon_y = 16;
-                                break;
-                              case GTK_ICON_SIZE_SMALL_TOOLBAR:
-                                icon_x = icon_y = 18;
-                                break;
-                              case GTK_ICON_SIZE_LARGE_TOOLBAR:
-                                icon_x = icon_y = 24;
-                                break;
-                              case GTK_ICON_SIZE_BUTTON:
-                                icon_x = icon_y = 20;
-                                break;
-                              case GTK_ICON_SIZE_DND:
-                                icon_x = icon_y = 32;
-                                break;
-                              case GTK_ICON_SIZE_DIALOG:
-                                icon_x = icon_y = 48;
-                                break;
-                              default:
-                                icon_x = icon_y = 0;
-                            }
-                          settings = gtk_settings_get_default ();
-                          if (settings)
-                            {
-                              gint w, h;
-
-                              if (gtk_icon_size_lookup_for_settings (settings,
-                                                            icon_size, &w, &h))
-                                {
-                                  icon_x = w;
-                                  icon_y = h;
-                                }
-                            }
-# if GLIB_CHECK_VERSION(2,6,0)
-                          sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
+                      sizes = gtk_icon_theme_get_icon_sizes (icon_theme,
                                                                     icon_name);
-                          if (sizes)
-                            {
-                              gint n = 0;
+                      if (sizes)
+                        {
+                          gint i, n = 0;
 
-                              for (i = 0; sizes[i] > 0; i++)
-                                if (ABS (icon_x - sizes[i])
+                          for (i = 0; sizes[i] > 0; i++)
+                            if (ABS (icon_x - sizes[i])
                                                     + ABS (icon_y - sizes[i])
                                         < ABS (icon_x - n) + ABS (icon_y - n))
-                                  n = sizes[i];
-                              g_free (sizes);
-                              if (n > 0)
-                                pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                                        icon_name, n, 0, NULL);
-                            }
-                          if (!pixbuf)
-# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                              n = sizes[i];
+                          g_free (sizes);
+                          if (n > 0)
                             pixbuf = gtk_icon_theme_load_icon (icon_theme,
-                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
+                                                        icon_name, n, 0, NULL);
                         }
+                      if (!pixbuf)
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+                        pixbuf = gtk_icon_theme_load_icon (icon_theme,
+                                    icon_name, (icon_x + icon_y) / 2, 0, NULL);
                     }
                 }
-              g_free (icon_name);
             }
+          g_free (icon_name);
         }
     }
+#endif /* GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
+#if defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE)
   return pixbuf;
-#else /* not G_OS_WIN32 && not MIMEFILE */
-  return NULL;
-#endif /* not G_OS_WIN32 && not MIMEFILE */
+#endif /* defined (G_OS_WIN32) || GLIB_CHECK_VERSION(2,16,0) || GLIB_CHECK_VERSION(2,4,0) && defined (MIMEFILE) */
 }
