@@ -1,10 +1,10 @@
 /*
     orzcomm
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "orzcomm.h"
 #include "misc/fileio.h"
@@ -36,12 +35,6 @@ enum
 #endif /* not G_OS_WIN32 */
 
 
-static void orz_comm_class_init (OrzCommClass *klass);
-static void orz_comm_init       (OrzComm      *comm);
-static void orz_comm_dispose    (GObject      *object);
-
-
-static GObjectClass *parent_class = NULL;
 static gint orz_comm_signals[LAST_SIGNAL] = {0};
 #ifdef G_OS_WIN32
 static GList *glist = NULL;
@@ -55,31 +48,8 @@ static volatile gboolean critical = FALSE;
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-GType
-orz_comm_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const static GTypeInfo info =
-      {
-        sizeof (OrzCommClass),
-        NULL,               /* base_init */
-        NULL,               /* base_finalize */
-        (GClassInitFunc)orz_comm_class_init,
-        NULL,               /* class_finalize */
-        NULL,               /* class_data */
-        sizeof (OrzComm),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc)orz_comm_init,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, "OrzComm", &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (OrzComm, orz_comm, G_TYPE_OBJECT);
+static void orz_comm_dispose (GObject *object);
 
 
 static void
@@ -87,9 +57,7 @@ orz_comm_class_init (OrzCommClass *klass)
 {
   GObjectClass *object_class;
 
-  parent_class = g_type_class_peek_parent (klass);
-  object_class = (GObjectClass *)klass;
-
+  object_class = G_OBJECT_CLASS (klass);
   object_class->dispose = orz_comm_dispose;
 
   klass->recv = NULL;
@@ -134,8 +102,9 @@ orz_comm_init (OrzComm *comm)
 static void
 orz_comm_dispose (GObject *object)
 {
-  OrzComm *comm = ORZ_COMM (object);
+  OrzComm *comm;
 
+  comm = ORZ_COMM (object);
 #ifdef USE_THREAD
   G_LOCK (critical);
   critical = TRUE;
@@ -167,8 +136,8 @@ orz_comm_dispose (GObject *object)
   G_UNLOCK (critical);
 #endif /* USE_THREAD */
 
-  if (G_OBJECT_CLASS (parent_class)->dispose)
-    G_OBJECT_CLASS (parent_class)->dispose (object);
+  if (G_OBJECT_CLASS (orz_comm_parent_class)->dispose)
+    G_OBJECT_CLASS (orz_comm_parent_class)->dispose (object);
 }
 
 
@@ -448,7 +417,7 @@ gboolean
 orz_comm_is_first (OrzComm *comm)
 {
 #ifdef G_OS_WIN32
-  return comm ? !comm->hConv :TRUE;
+  return comm ? !comm->hConv : TRUE;
 #else /* not G_OS_WIN32 */
   return comm ? comm->first : TRUE;
 #endif /* not G_OS_WIN32 */
@@ -464,7 +433,7 @@ orz_comm_send (OrzComm     *comm,
                const gint   argc,
                gchar      **argv)
 {
-  return orz_comm_send_with_files (comm, argc, argv, NULL);
+  orz_comm_send_with_files (comm, argc, argv, NULL);
 }
 
 
@@ -480,7 +449,7 @@ orz_comm_send_with_files (OrzComm         *comm,
                           const gboolean  *files)
 {
   gsize length = 0;
-  gchar *arg = NULL, *tmp;
+  gchar *arg = NULL;
   gint i;
 
 #ifdef G_OS_WIN32
@@ -491,10 +460,12 @@ orz_comm_send_with_files (OrzComm         *comm,
     return;
   for (i = 0; i < argc; i++)
     {
-      gint leng;
+      gsize leng;
 
       if (files && files[i])
         {
+          gchar *tmp;
+
           tmp = fileio_get_full_path (argv[i]);
           leng = g_strlen (tmp) * sizeof (gchar);
           arg = g_realloc (arg, length + leng + sizeof (gchar));
@@ -518,6 +489,7 @@ orz_comm_send_with_files (OrzComm         *comm,
       arg[length - 1] = '\0';
       hData = DdeCreateDataHandle (comm->ddeInst, arg, length, 0,
                                                 comm->hszTopic, CF_TEXT, 0);
+      g_free (arg);
       if (hData)
         DdeClientTransaction ((LPBYTE)hData, 0xffffffff, comm->hConv,
                     comm->hszTopic, CF_TEXT, XTYP_POKE, TIMEOUT_ASYNC, NULL);
@@ -527,8 +499,8 @@ orz_comm_send_with_files (OrzComm         *comm,
   G_LOCK (critical);
   critical = TRUE;
 # endif /* USE_THREAD */
-  tmp = comm->arg;
-  g_free (tmp);
+  g_free (comm->arg);
+  comm->arg = NULL;
 # ifdef USE_THREAD
   critical = FALSE;
   G_UNLOCK (critical);
