@@ -1,6 +1,6 @@
 /*
     Text maid
-    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2009 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -290,6 +290,7 @@ signal_commit (GtkIMContext *im_context,
   gtk_im_context_get_preedit_string (im_context, &str, NULL, NULL);
   if (g_strlen (str) <= 0)
     gtk_widget_hide (gtk_widget_get_parent (tmaid->preedit));
+  g_free (str);
 #endif /* not G_OS_WIN32 */
 }
 
@@ -307,16 +308,16 @@ signal_preedit_changed (GtkIMContext *im_context,
   widget = gtk_widget_get_parent (tmaid->preedit);
   if (g_strlen (str) > 0)
     {
-      int width;
+      int width, height;
 
       pango_layout_set_text (tmaid->layout, str, -1);
       pango_layout_set_attributes (tmaid->layout, attrs);
-      pango_layout_get_pixel_size (tmaid->layout, &width, NULL);
+      pango_layout_get_pixel_size (tmaid->layout, &width, &height);
       pango_layout_set_attributes (tmaid->layout, NULL);
-      gtk_window_resize (GTK_WINDOW (widget), width, tmaid->font_height);
+      height++;
+      gtk_window_resize (GTK_WINDOW (widget), width, height);
+      gtk_widget_queue_draw_area (tmaid->preedit, 0, 0, width, height);
       gtk_widget_show (widget);
-      gtk_widget_queue_draw_area (tmaid->preedit,
-                                            0, 0, width, tmaid->font_height);
     }
   else
     {
