@@ -30,6 +30,7 @@
 #include "menu.h"
 #include "panel.h"
 #include "print.h"
+#include "repinfo.h"
 #include "replace.h"
 #include "sigfile.h"
 #include "sigmain.h"
@@ -315,6 +316,8 @@ TMAID_COPYRIGHT_NOTICE"\n"
   gtk_box_pack_start (GTK_BOX (vbox), main_box, TRUE, TRUE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), table, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), find_box_get_widget (), FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (vbox),
+                                repinfo_box_get_widget (), FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), abort_box_get_widget (), FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -341,8 +344,9 @@ TMAID_COPYRIGHT_NOTICE"\n"
 
   /* ja:表示 */
   gtk_widget_show_all (window);
-  gtk_widget_hide (abort_box_get_widget ());
-  gtk_widget_hide (find_box_get_widget ());
+  gtk_widget_hide (abort_box);
+  gtk_widget_hide (find_box);
+  gtk_widget_hide (repinfo_box);
   gtk_widget_hide (label);
 
   /* ja:システム色 */
